# Copyright 2016 Brocade Communications System, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from django.utils.translation import gettext_lazy as _
from django.views.decorators.debug import sensitive_variables

from horizon import exceptions
from horizon import forms
from horizon import messages

from tacker_horizon.openstack_dashboard import api


class RegisterVim(forms.SelfHandlingForm):
    vim_type = forms.ChoiceField(choices=api.tacker.SUPPORTED_VIM_TYPES,
                                 label=_("VIM Type"))
    vim_name = forms.CharField(max_length=255, label=_("Name"))
    vim_description = forms.CharField(widget=forms.widgets.Textarea(
                                      attrs={'rows': 4}),
                                      label=_("Description"),
                                      required=False)
    auth_url = forms.URLField(label=_("Auth URL"))
    auth_method = forms.ChoiceField(widget=forms.widgets.RadioSelect,
                                    choices=api.tacker.AUTH_METHODS,
                                    initial="basic",
                                    label=_("Auth Method"))
    username = forms.CharField(max_length=80,
                               initial="admin",
                               label=_("Username"))
    password = forms.CharField(label=_("Password"),
                               widget=forms.PasswordInput(render_value=False))
    cert_verify = forms.ChoiceField(widget=forms.widgets.RadioSelect,
                                    choices=api.tacker.CERT_VERIFY_TYPES,
                                    initial="True",
                                    label=_("Cert Verify"))
    bearer_token = forms.CharField(widget=forms.widgets.Textarea(
                                   attrs={'rows': 4}),
                                   label=_("Bearer Token"))
    project_name = forms.CharField(max_length=80, label=_("Project Name"))
    domain_name = forms.CharField(max_length=80, label=_("Domain Name"),
                                  help_text=_('Applicable for OpenStack site '
                                              'running keystone v3. Run '
                                              'openstack domain list from '
                                              'CLI to find domain name'),
                                  required=False)
    ssl_ca_cert = forms.CharField(widget=forms.widgets.Textarea(
                                  attrs={'rows': 4}),
                                  label=_("SSL CA Certificate"),
                                  required=False)
    is_default = forms.BooleanField(
        label=_("Default"),
        initial=False,
        required=False,
        widget=forms.CheckboxInput(
            attrs={
                'class': 'switched',
            }
        )
    )

    def __init__(self, request, *args, **kwargs):
        super(RegisterVim, self).__init__(request, *args, **kwargs)

    def clean(self):
        data = super(RegisterVim, self).clean()
        return data

    @sensitive_variables('data', 'password')
    def handle(self, request, data):
        try:
            vim_name = data['vim_name']
            description = data['vim_description']
            password = data['password']
            username = data['username']
            project_name = data['project_name']
            is_default = data['is_default']
            auth_url = data['auth_url']
            domain_name = data['domain_name']
            cert_verify = data.get('cert_verify', api.tacker.CERT_TRUE_TYPE)
            if cert_verify not in [api.tacker.CERT_TRUE_TYPE,
                                   api.tacker.CERT_FALSE_TYPE]:
                raise forms.ValidationError("cert_verify type not supported.")
            auth_cred = {'username': username,
                         'password': password,
                         'user_domain_name': domain_name,
                         'cert_verify': cert_verify}
            bearer_token = data['bearer_token'].replace('None', '')
            ssl_ca_cert = data['ssl_ca_cert'].replace('\r\n', ' ')
            vim_type = data['vim_type']
            if vim_type == 'kubernetes':
                auth_cred = {'username': username,
                             'password': password}
                # if bearer_token is provided, use it instead
                if bearer_token:
                    auth_cred = {'bearer_token': bearer_token}
                # only k8s vim needs ssl_ca_cert and it's optional
                if ssl_ca_cert:
                    auth_cred['ssl_ca_cert'] = ssl_ca_cert
            vim_arg = {'vim': {'name': vim_name, 'description': description,
                               'type': vim_type, 'auth_url': auth_url,
                               'auth_cred': auth_cred,
                               'vim_project': {'name': project_name,
                                               'project_domain_name':
                                                   domain_name},
                               'is_default': is_default}}
            api.tacker.create_vim(request, vim_arg)
            messages.success(request,
                             _('VIM %s create operation initiated.') %
                             vim_name)
            return True
        except Exception:
            exceptions.handle(request,
                              _('Failed to register VIM.'))



# Copyright 2015 Brocade Communications System, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from django.utils.translation import gettext_lazy as _

import horizon


class Vnfmgroup(horizon.PanelGroup):
    slug = "nfvgroup"
    name = _("VNF Management")
    panels = ('vnfcatalog', 'vnfmanager',)


class Nfvogroup(horizon.PanelGroup):
    slug = "nfvogroup"
    name = _("NFV Orchestration")
    panels = ('vim', 'vnffgcatalog', 'vnffgmanager',
              'nscatalog', 'nsmanager')


class Nfv(horizon.Dashboard):
    name = _("NFV")
    slug = "nfv"
    panels = (Vnfmgroup, Nfvogroup,)  # Add your panels here.
    default_panel = 'vnfcatalog'  # Specify the slug of the dashboard's
    # default panel.


horizon.register(Nfv)



# Copyright 2016 Brocade Communications System, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from django.urls import reverse
from django.urls import reverse_lazy
from django.utils.translation import gettext_lazy as _

from horizon import exceptions
from horizon import forms
from horizon import tabs
from horizon.utils import memoized

from tacker_horizon.openstack_dashboard import api as tacker_api
from tacker_horizon.openstack_dashboard.dashboards.nfv.vim \
    import forms as project_forms

from tacker_horizon.openstack_dashboard.dashboards.nfv.vim \
    import tabs as vim_tabs


class IndexView(tabs.TabbedTableView):
    # A very simple class-based view...
    tab_group_class = vim_tabs.VIMTabs
    template_name = 'nfv/vim/index.html'

    def get_data(self, request, context, *args, **kwargs):
        # Add data to the context here...
        return context


class RegisterVIMView(forms.ModalFormView):
    form_class = project_forms.RegisterVim
    template_name = 'nfv/vim/registervim.html'
    success_url = reverse_lazy("horizon:nfv:vim:index")
    modal_id = "add_service_modal"
    modal_header = _("Register VIM")
    submit_label = _("Register VIM")
    submit_url = "horizon:nfv:vim:registervim"

    def get_context_data(self, **kwargs):
        context = super(RegisterVIMView, self).get_context_data(**kwargs)
        context['submit_url'] = reverse(self.submit_url)
        return context


class DetailView(tabs.TabView):
    tab_group_class = vim_tabs.VIMDetailsTabs
    template_name = 'nfv/vim/detail.html'
    redirect_url = 'horizon:nfv:vim:index'
    page_title = _("VIM Event Details: {{ vim_id }}")

    def get_context_data(self, **kwargs):
        context = super(DetailView, self).get_context_data(**kwargs)
        vim = self.get_data()
        context['vim'] = vim
        context['vim_id'] = kwargs['vim_id']
        context['url'] = reverse(self.redirect_url)
        return context

    @memoized.memoized_method
    def get_data(self):
        vim_id = self.kwargs['vim_id']

        try:
            vim = tacker_api.tacker.get_vim(self.request, vim_id)
            return vim
        except Exception:
            redirect = reverse(self.redirect_url)
            exceptions.handle(self.request,
                              _('Unable to retrieve details for '
                                'VIM "%s".') % vim_id,
                              redirect=redirect)
            raise exceptions.Http302(redirect)

    def get_tabs(self, request, *args, **kwargs):
        vim = self.get_data()
        return self.tab_group_class(request, vim=vim, **kwargs)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from django.utils.translation import gettext_lazy as _
from django.utils.translation import ngettext_lazy

from horizon import tables

from openstack_dashboard import policy
from tacker_horizon.openstack_dashboard import api


class MyFilterAction(tables.FilterAction):
    name = "myfilter"


class DeleteVNFFGD(policy.PolicyTargetMixin, tables.DeleteAction):
    @staticmethod
    def action_present(count):
        return ngettext_lazy(
            "Delete VNFFG",
            "Delete VNFFGs",
            count
        )

    @staticmethod
    def action_past(count):
        return ngettext_lazy(
            "Delete VNFFG",
            "Delete VNFFGs",
            count
        )

    def action(self, request, obj_id):
        api.tacker.delete_vnffgd(request, obj_id)


class OnBoardVNFFG(tables.LinkAction):
    name = "onboardvnffg"
    verbose_name = _("Onboard VNFFG")
    classes = ("ajax-modal",)
    icon = "plus"
    url = "horizon:nfv:vnffgcatalog:onboardvnffg"


class VNFFGCatalogTable(tables.DataTable):
    name = tables.Column('name',
                         link="horizon:nfv:vnffgcatalog:detail",
                         verbose_name=_("Name"))
    description = tables.Column('description',
                                verbose_name=_("Description"))
    id = tables.Column('id',
                       verbose_name=_("Catalog Id"))

    class Meta(object):
        name = "vnffgcatalog"
        verbose_name = _("VNFFGCatalog")
        table_actions = (OnBoardVNFFG, DeleteVNFFGD, MyFilterAction,)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from django.utils.translation import gettext_lazy as _
from oslo_log import log as logging

from horizon import exceptions
from horizon import forms
from horizon import messages

from tacker_horizon.openstack_dashboard import api

LOG = logging.getLogger(__name__)


class DeployVNFFG(forms.SelfHandlingForm):
    vnffg_name = forms.CharField(max_length=255, label=_("VNFFG Name"))
    vnffgd_id = forms.ChoiceField(label=_("VNFFG Catalog Name"))
    vnf_mapping = forms.CharField(label=_("VNF Mapping"),
                                  required=False)
    symmetrical = forms.BooleanField(label=_("Symmetrical path"),
                                     required=False, help_text="Decides"
                                     " whether to automatically create a"
                                     " reverse patch for the NFP")

    def __init__(self, request, *args, **kwargs):
        super(DeployVNFFG, self).__init__(request, *args, **kwargs)

        try:
            vnffgd_list = api.tacker.vnffgd_list(request)
            available_choices_vnffgd = [(vnffg['id'], vnffg['name']) for
                                        vnffg in vnffgd_list]
        except Exception as e:
            available_choices_vnffgd = []
            msg = _('Failed to retrieve available VNFFG Catalog names: %s') % e
            LOG.error(msg)

        self.fields['vnffgd_id'].choices = [('', _('Select a VNFFG Catalog '
                                            'Name'))]+available_choices_vnffgd

    def clean(self):
        data = super(DeployVNFFG, self).clean()
        return data

    def list_vnfs(self, request):
        vnfs = [vnfs['name'] for vnfs in api.tacker.vnf_list(request)]
        return vnfs

    def handle(self, request, data):
        try:
            vnffg_name = data['vnffg_name']
            vnffgd_id = data['vnffgd_id']
            vnf_mapping = data['vnf_mapping']
            _vnf_mapping = dict()
            if vnf_mapping:
                _vnf_mappings = vnf_mapping.split(",")
                vnfs = self.list_vnfs(request)
                for mapping in _vnf_mappings:
                    vnfd_name, vnf = mapping.split(":", 1)
                    try:
                        if vnf in vnfs:
                            _vnf_mapping[vnfd_name] = vnf
                        else:
                            raise Exception
                    except Exception:
                        exceptions.handle(request, _('Specified VNF %s not'
                                                     ' found') % vnf)

            symmetrical = data['symmetrical']
            vnffg_arg = {'vnffg': {'vnffgd_id': vnffgd_id,
                                   'name': vnffg_name,
                                   'symmetrical': symmetrical,
                                   'vnf_mapping': _vnf_mapping}}

            api.tacker.create_vnffg(request, vnffg_arg)
            messages.success(request,
                             _('VNFFG %s create operation initiated.') %
                             vnffg_name)
            return True
        except Exception:
            exceptions.handle(request,
                              _('Failed to create VNFFG.'))



# Copyright 2015 Brocade Communications System, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import yaml

from django.forms import ValidationError
from django.utils.translation import gettext_lazy as _
from oslo_log import log as logging

from horizon import exceptions
from horizon import forms
from horizon import messages

from tacker_horizon.openstack_dashboard import api

LOG = logging.getLogger(__name__)


class DeployVNF(forms.SelfHandlingForm):
    vnf_name = forms.CharField(max_length=255, label=_("VNF Name"))
    description = forms.CharField(widget=forms.widgets.Textarea(
                                  attrs={'rows': 4}),
                                  label=_("Description"),
                                  required=False)
    vnfd_id = forms.ChoiceField(label=_("VNF Catalog Name"),
                                required=False)
    template_source = forms.ChoiceField(
        label=_('VNFD template Source'),
        required=False,
        choices=[('file', _('File')),
                 ('raw', _('Direct Input'))],
        widget=forms.Select(
            attrs={'class': 'switchable', 'data-slug': 'template'}))
    template_file = forms.FileField(
        label=_('VNFD template File'),
        help_text=_('VNFD template to create VNF'),
        widget=forms.FileInput(
            attrs={'class': 'switched', 'data-switch-on': 'template',
                   'data-template-file': _('TOSCA Template File')}),
        required=False)
    template_input = forms.CharField(
        label=_('VNFD template'),
        help_text=_('The YAML formatted contents of VNFD template.'),
        widget=forms.widgets.Textarea(
            attrs={'class': 'switched', 'data-switch-on': 'template',
                   'data-template-raw': _('VNFD template')}),
        required=False)
    vim_id = forms.ChoiceField(label=_("VIM Name"), required=False)
    region_name = forms.CharField(label=_("Region Name"), required=False)
    source_type = forms.ChoiceField(
        label=_('Parameter Value Source'),
        required=False,
        choices=[('file', _('File')),
                 ('raw', _('Direct Input'))],
        widget=forms.Select(
            attrs={'class': 'switchable', 'data-slug': 'source'}))

    param_file = forms.FileField(
        label=_('Parameter Value File'),
        help_text=_('A local Parameter Value file to upload.'),
        widget=forms.FileInput(
            attrs={'class': 'switched', 'data-switch-on': 'source',
                   'data-source-file': _('Parameter Value File')}),
        required=False)

    direct_input = forms.CharField(
        label=_('Parameter Value YAML'),
        help_text=_('The YAML formatted contents of Parameter Values.'),
        widget=forms.widgets.Textarea(
            attrs={'class': 'switched', 'data-switch-on': 'source',
                   'data-source-raw': _('Parameter Values')}),
        required=False)

    config_type = forms.ChoiceField(
        label=_('Configuration Value Source'),
        required=False,
        choices=[('file', _('File')),
                 ('raw', _('Direct Input'))],
        widget=forms.Select(
            attrs={'class': 'switchable', 'data-slug': 'config'}))

    config_file = forms.FileField(
        label=_('Configuration Value File'),
        help_text=_('VNF Configuration file with YAML '
                    'formatted contents to upload.'),
        widget=forms.FileInput(
            attrs={'class': 'switched', 'data-switch-on': 'config',
                   'data-config-file': _('Configuration Value File')}),
        required=False)

    config_input = forms.CharField(
        label=_('Configuration Value YAML'),
        help_text=_('YAML formatted VNF configuration text.'),
        widget=forms.widgets.Textarea(
            attrs={'class': 'switched', 'data-switch-on': 'config',
                   'data-config-raw': _('Configuration Values')}),
        required=False)

    def __init__(self, request, *args, **kwargs):
        super(DeployVNF, self).__init__(request, *args, **kwargs)

        try:
            vnfd_list = api.tacker.vnfd_list(request,
                                             template_source='onboarded')
            available_choices_vnfd = [(vnf['id'], vnf['name']) for vnf in
                                      vnfd_list]
        except Exception as e:
            available_choices_vnfd = []
            msg = _('Failed to retrieve available VNF Catalog names: %s') % e
            LOG.error(msg)

        try:
            vim_list = api.tacker.vim_list(request)
            available_choices_vims = [(vim['id'], vim['name']) for vim in
                                      vim_list]

        except Exception as e:
            available_choices_vims = []
            msg = _('Failed to retrieve available VIM names: %s') % e
            LOG.error(msg)

        self.fields['vnfd_id'].choices = [('', _('Select a VNF Catalog Name'))
                                          ]+available_choices_vnfd
        self.fields['vim_id'].choices = [('',
                                          _('Select a VIM Name'))
                                         ]+available_choices_vims

    def clean(self):
        data = super(DeployVNF, self).clean()

        template_file = data.get('template_file', None)
        template_raw = data.get('template_input', None)

        if template_raw and template_file:
            raise ValidationError(
                _("Cannot specify both file and direct input."))

        if template_file and not template_file.name.endswith('.yaml'):
            raise ValidationError(
                _("Please upload .yaml file only."))

        if template_file:
            data['vnfd_template'] = yaml.load(template_file,
                                              Loader=yaml.SafeLoader)
        elif template_raw:
            data['vnfd_template'] = yaml.load(data['template_input'],
                                              Loader=yaml.SafeLoader)
        else:
            data['vnfd_template'] = None

        param_file = data.get('param_file', None)
        param_raw = data.get('direct_input', None)

        if param_raw and param_file:
            raise ValidationError(
                _("Cannot specify both file and direct input."))

        if param_file and not param_file.name.endswith('.yaml'):
            raise ValidationError(
                _("Please upload .yaml file only."))

        if param_file:
            data['param_values'] = self.files['param_file'].read()
        elif param_raw:
            data['param_values'] = data['direct_input']
        else:
            data['param_values'] = None

        config_file = data.get('config_file', None)
        config_raw = data.get('config_input', None)

        if config_file and config_raw:
            raise ValidationError(
                _("Cannot specify both file and direct input."))

        if config_file and not config_file.name.endswith('.yaml'):
            raise ValidationError(_("Only .yaml file uploads supported"))

        if config_file:
            data['config_values'] = self.files['config_file'].read()
        elif config_raw:
            data['config_values'] = data['config_input']
        else:
            data['config_values'] = None

        return data

    def handle(self, request, data):
        try:
            vnf_name = data['vnf_name']
            description = data['description']
            vnfd_id = data.get('vnfd_id')
            vnfd_template = data.get('vnfd_template')
            vim_id = data['vim_id']
            region_name = data['region_name']
            param_val = data['param_values']
            config_val = data['config_values']

            if (vnfd_id == '') and (vnfd_template is None):
                raise ValidationError(_("Both VNFD id and template cannot be "
                                        "empty. Please specify one of them"))

            if (vnfd_id != '') and (vnfd_template is not None):
                raise ValidationError(_("Both VNFD id and template cannot be "
                                        "specified. Please specify any one"))

            vnf_arg = {'vnf': {'vnfd_id': vnfd_id, 'name':  vnf_name,
                               'description': description,
                               'vim_id': vim_id,
                               'vnfd_template': vnfd_template}}
            if region_name:
                vnf_arg.setdefault('placement_attr', {})[
                    region_name] = region_name
            vnf_attr = vnf_arg['vnf'].setdefault('attributes', {})
            if param_val:
                vnf_attr['param_values'] = param_val
            if config_val:
                vnf_attr['config'] = config_val

            api.tacker.create_vnf(request, vnf_arg)
            messages.success(request,
                             _('VNF %s create operation initiated.') %
                             vnf_name)
            return True
        except Exception:
            exceptions.handle(request,
                              _('Failed to create VNF.'))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from django.http import Http404
from django.utils.translation import gettext_lazy as _
from django.utils.translation import ngettext_lazy
from django.utils.translation import pgettext_lazy

from horizon import messages
from horizon import tables

from openstack_dashboard import policy
from tacker_horizon.openstack_dashboard import api
from tackerclient.common.exceptions import NotFound


class VNFFGManagerItem(object):
    def __init__(self, id, name, description, status):
        self.id = id
        self.name = name
        self.description = description
        self.status = status


class VNFFGManagerItemList(object):
    VNFFGLIST_P = []

    @classmethod
    def get_obj_given_id(cls, vnffg_id):
        for obj in cls.VNFFGLIST_P:
            if obj.id == vnffg_id:
                return obj

    @classmethod
    def add_item(cls, item):
        cls.VNFFGLIST_P.append(item)

    @classmethod
    def clear_list(cls):
        cls.VNFFGLIST_P = []


class MyFilterAction(tables.FilterAction):
    name = "myfilter"


class VNFFGUpdateRow(tables.Row):
    ajax = True

    def can_be_selected(self, datum):
        return datum.status != 'DELETE_COMPLETE'

    def get_data(self, request, vnffg_id):
        try:
            item = VNFFGManagerItemList.get_obj_given_id(vnffg_id)
            vnffg_instance = api.tacker.get_vnffg(request, vnffg_id)

            if not vnffg_instance and not item:
                # TODO(NAME) - bail with error
                return None

            if not vnffg_instance and item:
                # API failure, just keep the current state
                return item

            vnffg = vnffg_instance['vnffg']
            try:
                vnffg_desc_str = vnffg['description']
            except KeyError:
                vnffg_desc_str = ""

            if not item:
                # Add an item entry
                item = VNFFGManagerItem(vnffg['name'], vnffg_desc_str,
                                        vnffg['status'], vnffg['id'])
            else:
                item.description = vnffg_desc_str
                item.status = vnffg['status']
            return item
        except (Http404, NotFound):
            raise Http404
        except Exception as e:
            messages.error(request, e)
            raise


class DeleteVNFFG(policy.PolicyTargetMixin, tables.DeleteAction):
    @staticmethod
    def action_present(count):
        return ngettext_lazy(
            "Terminate VNFFG",
            "Terminate VNFFGs",
            count
        )

    @staticmethod
    def action_past(count):
        return ngettext_lazy(
            "Terminate VNFFG",
            "Terminate VNFFGs",
            count
        )

    def action(self, request, obj_id):
        api.tacker.delete_vnffg(request, obj_id)


class DeployVNFFG(tables.LinkAction):
    name = "deployvnffg"
    verbose_name = _("Deploy VNFFG")
    classes = ("ajax-modal",)
    icon = "plus"
    url = "horizon:nfv:vnffgmanager:deployvnffg"


class VNFFGManagerTable(tables.DataTable):
    STATUS_CHOICES = (
        ("ACTIVE", True),
        ("ERROR", False),
    )
    STACK_STATUS_DISPLAY_CHOICES = (
        ("init_in_progress", pgettext_lazy("current status of stack",
                                           "Init In Progress")),
        ("init_complete", pgettext_lazy("current status of stack",
                                        "Init Complete")),
        ("init_failed", pgettext_lazy("current status of stack",
                                      "Init Failed")),
        ("create_in_progress", pgettext_lazy("current status of stack",
                                             "Create In Progress")),
        ("create_complete", pgettext_lazy("current status of stack",
                                          "Create Complete")),
        ("create_failed", pgettext_lazy("current status of stack",
                                        "Create Failed")),
        ("delete_in_progress", pgettext_lazy("current status of stack",
                                             "Delete In Progress")),
        ("delete_complete", pgettext_lazy("current status of stack",
                                          "Delete Complete")),
        ("delete_failed", pgettext_lazy("current status of stack",
                                        "Delete Failed")),
        ("update_in_progress", pgettext_lazy("current status of stack",
                                             "Update In Progress")),
        ("update_complete", pgettext_lazy("current status of stack",
                                          "Update Complete")),
        ("update_failed", pgettext_lazy("current status of stack",
                                        "Update Failed")),
        ("rollback_in_progress", pgettext_lazy("current status of stack",
                                               "Rollback In Progress")),
        ("rollback_complete", pgettext_lazy("current status of stack",
                                            "Rollback Complete")),
        ("rollback_failed", pgettext_lazy("current status of stack",
                                          "Rollback Failed")),
        ("suspend_in_progress", pgettext_lazy("current status of stack",
                                              "Suspend In Progress")),
        ("suspend_complete", pgettext_lazy("current status of stack",
                                           "Suspend Complete")),
        ("suspend_failed", pgettext_lazy("current status of stack",
                                         "Suspend Failed")),
        ("resume_in_progress", pgettext_lazy("current status of stack",
                                             "Resume In Progress")),
        ("resume_complete", pgettext_lazy("current status of stack",
                                          "Resume Complete")),
        ("resume_failed", pgettext_lazy("current status of stack",
                                        "Resume Failed")),
        ("adopt_in_progress", pgettext_lazy("current status of stack",
                                            "Adopt In Progress")),
        ("adopt_complete", pgettext_lazy("current status of stack",
                                         "Adopt Complete")),
        ("adopt_failed", pgettext_lazy("current status of stack",
                                       "Adopt Failed")),
        ("snapshot_in_progress", pgettext_lazy("current status of stack",
                                               "Snapshot In Progress")),
        ("snapshot_complete", pgettext_lazy("current status of stack",
                                            "Snapshot Complete")),
        ("snapshot_failed", pgettext_lazy("current status of stack",
                                          "Snapshot Failed")),
        ("check_in_progress", pgettext_lazy("current status of stack",
                                            "Check In Progress")),
        ("check_complete", pgettext_lazy("current status of stack",
                                         "Check Complete")),
        ("check_failed", pgettext_lazy("current status of stack",
                                       "Check Failed")),
    )
    name = tables.Column("name",
                         link="horizon:nfv:vnffgmanager:detail",
                         verbose_name=_("VNFFG Name"))
    description = tables.Column("description",
                                verbose_name=_("Description"))
    status = tables.Column("status",
                           hidden=False,
                           status=True,
                           status_choices=STATUS_CHOICES)

    class Meta(object):
        name = "vnffgmanager"
        verbose_name = _("VNFFGManager")
        status_columns = ["status", ]
        row_class = VNFFGUpdateRow
        table_actions = (DeployVNFFG, DeleteVNFFG, MyFilterAction,)
